import type { VxeGridProps } from '#/adapter/vxe-table';

import { h } from 'vue';

import { DictEnum } from '@vben/constants';
import { FolderIcon, MenuIcon, VbenIcon } from '@vben/icons';
import { getPopupContainer } from '@vben/utils';

import { type FormSchemaGetter } from '#/adapter/form';
import { getDictOptions } from '#/utils/dict';
import { renderDict } from '#/utils/render';

export const querySchema: FormSchemaGetter = () => [
  {
    component: 'Input',
    fieldName: 'typeName',
    label: '类型名称 '
  },
  {
    component: 'Select',
    componentProps: {
      getPopupContainer,
      options: getDictOptions(DictEnum.SYS_NORMAL_DISABLE)
    },
    fieldName: 'status',
    label: '类型状态 '
  }
];

// 菜单类型（M目录 C菜单 F按钮）
export const fixtureTypeOptions = [
  { label: '目录', value: 'M' },
  { label: '类型', value: 'C' }
];

export const yesNoOptions = [
  { label: '是', value: '0' },
  { label: '否', value: '1' }
];

// （M目录 C菜单）
const menuTypes = {
  M: { icon: FolderIcon, value: '目录' },
  C: { icon: MenuIcon, value: '类型' }
};
export const columns: VxeGridProps['columns'] = [
  {
    title: '类型名称',
    field: 'typeName',
    treeNode: true,
    width: 200
  },
  {
    title: '类型编码',
    field: 'typeCode',
    width: 200
  },
  {
    title: '图标',
    field: 'icon',
    width: 80,
    slots: {
      default: ({ row }) => {
        if (row?.icon === '#') {
          return '';
        }
        return (
          <span class={'flex justify-center'}>
            <VbenIcon icon={row.icon} />
          </span>
        );
      }
    }
  },
  {
    title: '排序',
    field: 'orderNum',
    width: 120
  },
  {
    title: '节点类型',
    field: 'menuType',
    width: 150,
    slots: {
      default: ({ row }) => {
        const current = menuTypes[row.menuType as 'C' | 'M'];
        if (!current) {
          return '未知';
        }
        return (
          <span class="flex items-center justify-center gap-1">
            {h(current.icon, { class: 'size-[18px]' })}
            <span>{current.value}</span>
          </span>
        );
      }
    }
  },
  {
    title: '状态',
    field: 'status',
    width: 100,
    slots: {
      default: ({ row }) => {
        return renderDict(row.status, DictEnum.SYS_NORMAL_DISABLE);
      }
    }
  },
  {
    title: '创建时间',
    field: 'createTime'
  },
  {
    field: 'action',
    fixed: 'right',
    slots: { default: 'action' },
    title: '操作',
    width: 200
  }
];

export const drawerSchema: FormSchemaGetter = () => [
  {
    component: 'Input',
    dependencies: {
      show: () => false,
      triggerFields: ['']
    },
    fieldName: 'id'
  },
  {
    component: 'TreeSelect',
    defaultValue: 0,
    fieldName: 'parentId',
    label: '上级菜单',
    rules: 'selectRequired'
  },
  {
    component: 'RadioGroup',
    componentProps: {
      buttonStyle: 'solid',
      options: fixtureTypeOptions,
      optionType: 'button'
    },
    defaultValue: 'M',
    dependencies: {
      componentProps: (_, api) => {
        // 切换时清空校验
        // 直接抄的源码 没有清空校验的方法
        Object.keys(api.errors.value).forEach((key) => {
          api.setFieldError(key, undefined);
        });
        return {};
      },
      triggerFields: ['menuType']
    },
    fieldName: 'menuType',
    label: '节点类型'
  },
  {
    component: 'Input',
    fieldName: 'typeName',
    label: '类型名称',
    rules: 'required'
  },
  {
    component: 'Input',
    fieldName: 'typeCode',
    label: '类型编码',
    rules: 'required'
  },
  {
    component: 'InputNumber',
    fieldName: 'orderNum',
    help: '排序, 数字越小越靠前',
    label: '显示排序',
    rules: 'required'
  },
  {
    component: 'RadioGroup',
    componentProps: {
      buttonStyle: 'solid',
      options: getDictOptions(DictEnum.SYS_NORMAL_DISABLE),
      optionType: 'button'
    },
    defaultValue: '0',
    dependencies: {
      // 类型不为按钮时显示
      show: (values) => values.menuType !== 'F',
      triggerFields: ['menuType']
    },
    fieldName: 'status',
    help: '停用后不会出现在列表',
    label: '类型状态'
  }
];
